library(compositions)
library(phylofactor)
library(ape)
library(phangorn)
library(phylofactor)
source('C:/Users/Big Alculus/Documents/Boulder/PhyloStats/R files/PhyloStats.R')


####################################### PercentCorrect ############################################
PercentCorrect <- function(effect,sample.size,reps,tree,method='ILR',TreeV=NULL,Groups=NULL,nclades=1,choice='var'){
  #input effect size, n, and method for regression in c("ILR", "PhyloFactor")
  #output the percent of regressions which correctly ID'd the clade of interest
  
  if (method != 'PhyloFactor'){if (is.null(TreeV)){TreeV <- tree.ilr(tree)$TreeILR}}
  
  n <- length(tree$tip.label)
  ILR.to.node <- apply(TreeV,MARGIN = 2,FUN=function(v,tree){return(ILRtoNode(tree,v))},tree=tree)
  perc.correct = rep(0,length(sample.size))
  val <- 0
  
  if (nclades>1 && length(effect)!=nclades){
    warning('input "effect" is not of length nclades - choosing random effects as lnormal random variables')
    effect <- rlnorm(nclades)
  }
  
  for (p in sample.size){
    val = val+1
    CorrectClade <- rep(0,reps)
    X <- as.factor(c(rep(1,p),rep(2,p)))
    for (rr in 1:reps){
      Groupn <- sample(length(Groups),nclades)  #Groups[[1]] is all of the taxa in the tree - hence, it's excluded to ensure one sub-tree responds differently to X.
      
      Abundances <- matrix(round(rlnorm(2*p*n,meanlog=8,sdlog=.5)),nrow=2*p)
      sigClades <- Groups[Groupn] #significant clades
      sigClades <- lapply(sigClades,sort)
      for (cc in 1:nclades){
        Abundances[1:p,sigClades[[cc]]] <- Abundances[1:p,sigClades[[cc]]]*effect[cc] #The effects of significant Clades
      }
      Rc <- clo(Abundances)
      colnames(Rc) <- tree$tip.label
      Clade <- rep(NA,nclades)
      if (method=='PhyloFactor'){
        PF <- PhyloFactor(Data=t(Rc),tree=tree,X=X,nfactors=nclades,choice=choice)
        # Clade <- as.numeric(phyFacts$Nodes)
        
        if (nclades>1){
        correct <- apply(PF$basis,2,FUN = function(x) which(x>0)) %in% sigClades
        correct <- mapply(as.list(correct),FUN=function(a,b) a | b, as.list(apply(PF$basis,2,FUN = function(x) which(x<0)) %in% sigClades))
        } else {
          correct <- (list(which(PF$basis<0)) %in% sigClades) | list(which(PF$basis>0)) %in% sigClades
        }
        # CorrectClade[rr] = sum(Clade %in% Groupn)/nclades
        correct <- sum(correct)
        CorrectClade[rr] = sum(correct)/nclades
      } else {
  
        for (cc in 1:nclades){
          rootedILR <- ilr(Rc,TreeV)
          # Map the most significant ILR coordinate to node in tree
          if (choice=='F'){
            pvals <- apply(rootedILR,MARGIN=2,FUN=function(y,x){return(summary(aov(y~x))[[1]][1,'Pr(>F)'])},x=X)
            nn=which(pvals==min(pvals))
          } else {
            glms <- apply(rootedILR,2,FUN=function(y,x) glm(y~x),x=X)
            ex.var <- sapply(glms,FUN=function(gg) (gg$null.deviance-gg$deviance)/length(gg$residuals))
            nn <- which(ex.var==max(ex.var))
          }
          Clade[cc] <- ILR.to.node[nn]
          
          #regress out the effect of dominant ILR
          dum <- matrix(0,dim(rootedILR)[1],dim(rootedILR)[2])
          dum[,nn] <- predict(glm(rootedILR[,nn]~X))
          Rc <- clrInv(clr(Rc)- clr(ilrInv(dum,V=TreeV)))
        }
        #the "right Answer" for the tILR would be the immediate ancestral node of each sigClades, identified by node Groupn
        
        rightAns <- unlist(lapply(as.list(Groupn),function(grp,tree,rooot){return(nodepath(tree,grp,rooot)[2])},tree=tree,rooot=n+1))
        CorrectClade[rr] = sum(Clade %in% rightAns)/nclades
      }
    }
    perc.correct[val] <- sum(CorrectClade)/length(CorrectClade)
  }
  return(perc.correct)
}



set.seed(1)
n=50
phylo <- rtree(n)
tree <- chronopl(phylo,lambda=1)
###################################### Power Analysis #################################################
#################################### 1 clade, vary effects and sample.sizes ###########################
effects <- as.list(c(1.5,3,6))
sample.sizes <- c(2,4,6,8,10,15,30,45,80,120,150)
# sample.sizes <- c(seq(2,10,by=2),seq(12,40,by=7),seq(50,150,by=25))
reps=1000
Groups <- c(as.list(1:n),clade.members.list(phylo))
TreeV <- tree.ilr(tree)$TreeILR


timestamp()
cl <- phyloFcluster(3)
clusterEvalQ(cl,source('C:/Users/Big Alculus/Documents/Boulder/PhyloStats/R files/PhyloStats.R'))
PowerAnalysis <- NULL
PowerAnalysis$rooted <- parLapply(cl,effects,fun=PercentCorrect,sample.size=sample.sizes,reps=reps,tree=tree,method='ILR',TreeV=TreeV,Groups=Groups)
stopCluster(cl)

cl <- phyloFcluster(3)
clusterEvalQ(cl,source('C:/Users/Big Alculus/Documents/Boulder/PhyloStats/R files/PhyloStats.R'))
PowerAnalysis$phyFact <- parLapply(cl,effects,fun=PercentCorrect,sample.size=sample.sizes,reps=reps,tree=tree,method='PhyloFactor',TreeV=TreeV,Groups=Groups)
stopCluster(cl)
timestamp()


save(list=ls(),file = 'C:/Users/Big Alculus/Documents/Boulder/PhyloStats/PhyloFactor Analysis Workspace 5')


## Now we plot and label. First, we'll get these as matrices.

RDat <- matrix(unlist(PowerAnalysis$rooted),nrow=length(effects),byrow=T)
PDat <- matrix(unlist(PowerAnalysis$phyFact),nrow=length(effects),byrow=T)

tiff(filename = 'C:/Users/Big Alculus/Documents/Boulder/PhyloStats/Paper Figures/Power-Analysis_1_clade.tiff',width=1000,height=500)
par(mfrow=c(1,2))
cols <- rainbow(length(effects))
lty = rev(1:length(effects))
pchs = c(0,1,15,19)
matplot(sample.sizes,t(RDat),type='l',main='Power Analysis - Rooted ILR',ylim=c(.1,1),col = cols,lwd=3,lty = lty, pch = lty, cex=5,xlab='Sample Size',ylab='Accuracy - %Correct Clade ID')
for (ll in 1:dim(RDat)[1]){
  points(sample.sizes,RDat[ll,],pch=pchs[ll],col=cols[ll],cex=2)
}
### label 
lbls=lapply(as.list(paste(effects)),FUN=function(x,y){paste(y,x)},y=c('a ='))
legend(4,1,unlist(lbls),lty = lty,pch=pchs,lwd=2,col=cols)

matplot(sample.sizes,t(PDat),type='l',main='Power Analysis - PhyloFactor ILR',ylim=c(.1,1),col = cols,lwd=3,lty = lty,xlab='Sample Size',ylab='Accuracy - %Correct Clade ID')
for (ll in 1:dim(RDat)[1]){
  points(sample.sizes,PDat[ll,],pch=pchs[ll],col=cols[ll],cex=2)
}

dev.off()



#########################################################################################################
#########################################################################################################
########################################## K=3 clades #####################################################

### Now we'll repeat the above power analysis but with multiple clades. 

#Now, we want to make our "effects" be a list of vectors. 
nclades= 3
neffects = 5 #the number of sets of effects we'll have.
effect <- c(2/pi,3/4,exp(1)/2) #chosen to be irrational multiples of one-another
effects <- lapply(as.list(1:neffects),function(x,y){return(y^x)},y=effect)
sample.sizes <- c(2,4,8,16,32,64)
reps=3
TreeV <- tree.ilr(tree)$TreeILR


cl <- phyloFcluster(3)
clusterEvalQ(cl,source('C:/Users/Big Alculus/Documents/Boulder/PhyloStats/R files/PhyloStats.R'))

PowerAnalysisK <- NULL
PowerAnalysisK$rooted <- parLapply(cl,effects,fun=PercentCorrect,sample.size=sample.sizes,reps=reps,tree=tree,method='ILR',TreeV=TreeV,Groups=Groups,nclades=nclades)
stopCluster(cl)
cl <- phyloFcluster(3)
clusterEvalQ(cl,source('C:/Users/Big Alculus/Documents/Boulder/PhyloStats/R files/PhyloStats.R'))

PowerAnalysisK$phyFact <- parLapply(cl,effects,fun=PercentCorrect,sample.size=sample.sizes,reps=reps,tree=tree,method='PhyloFactor',TreeV=TreeV,Groups=Groups,nclades=nclades)
stopCluster(cl)


save(list=ls(),file = 'PhyloFactor Analysis Workspace - including K clades analysis')


## Now we plot and label. First, we'll get these as matrices.

RDatK <- matrix(unlist(PowerAnalysisK$rooted),nrow=length(effects),byrow=T)
PDatK <- matrix(unlist(PowerAnalysisK$phyFact),nrow=length(effects),byrow=T)

par(mfrow=c(1,2))
cols <- rainbow(length(effects))
lty = rev(1:length(effects))
pchs = c(0,1,15,19)
matplot(sample.sizes,t(RDatK),type='l',main='Power Analysis - Rooted ILR',ylim=c(.1,1),col = cols,lwd=3,lty = lty, pch = lty, cex=5,xlab='Sample Size',ylab='Accuracy - %Correct Clade ID')
for (ll in 1:dim(RDatK)[1]){
  points(sample.sizes,RDatK[ll,],pch=pchs[ll],col=cols[ll],cex=2)
}
### label 
lbls=lapply(as.list(paste(as.list(1:neffects))),FUN=function(x,y){paste(y,x)},y=c('b ='))
legend(4,1,unlist(lbls),lty = lty,pch=pchs,lwd=2,col=cols)

matplot(sample.sizes,t(PDatK),type='l',main='Power Analysis - PhyloFactor ILR',ylim=c(.1,1),col = cols,lwd=3,lty = lty,xlab='Sample Size',ylab='Accuracy - %Correct Clade ID')
for (ll in 1:dim(RDatK)[1]){
  points(sample.sizes,PDatK[ll,],pch=pchs[ll],col=cols[ll],cex=2)
}

############################### How well can PhyloFactor determine the number of clades? #################

CladeCounter <- function(NClades,tree,Groups,effectSDlog = 3,n=50,p=16){
  output <- rep(0,length(NClades))
  X <- as.factor(c(rep(1,p),rep(2,p)))
  for (rr in 1:length(NClades)){
    nclades <- NClades[rr]
    effect <- rlnorm(nclades,sdlog=effectSDlog)
    Groupn <- sample(1:length(Groups),nclades)  #Groups[[1]] is all of the taxa in the tree - hence, it's excluded to ensure one sub-tree responds differently to X.
    Abundances <- matrix(round(rlnorm(2*p*n,meanlog=8,sdlog=.5)),nrow=2*p)
    sigClades <- Groups[Groupn-n] #significant clades
    for (cc in 1:nclades){
      site <- sample(X,1)
      site <- which(X==site)
      Abundances[site,sigClades[[cc]]] <- Abundances[site,sigClades[[cc]]]*effect[cc] #The effects of significant Clades
    }
    Rc <- clo(Abundances)
    colnames(Rc) <- tree$tip.label
    
    output[rr] <- PhyloFactor(t(Rc),tree,X,stop.early=T)$nfactors
  }
  output <- matrix(c(output,NClades),ncol=2,byrow=F)
  colnames(output) <- c('PF nfactors','True nfactors')
  return(output)
}


reps = 300 #Number of phylofactorizations we'll perform for each N
N=10 #maximum number of clades we'll effect
NClades <- rep(1:N,times=reps)

ncores=3
cl <- phyloFcluster(ncores)
pNClades <- clusterSplit(cl,NClades)
Clades <- parLapply(cl,X = pNClades,fun = CladeCounter,tree=tree,Groups=Groups)
stopCluster(cl)


#Cleaning & extracting data from parLapply
PF.NClades <- data.frame('PF.nfactors'=rep(NA,reps*N),'True.nfactors'=rep(NA,reps*N))
for (pp in 1:ncores){
  if (pp==1){
    ix=1:nrow(Clades[[pp]])
  } else {
    ix <- ix+nrow(Clades[[pp]])
  }
  PF.NClades[ix,] <- Clades[[pp]]
}
colnames(PF.NClades) <- c('PF.nfactors','True.nfactors')

#Linear regression of PF.nfactors~True.nfactors
best.fit <- glm(PF.nfactors~True.nfactors,family='Poisson',data=PF.NClades)
fitted <- predict(best.fit,interval='confidence')

#Plotting the best fit - did stop.fcn=T get, on average, the correct number of clades?
plot(1:N,1:N,type='l',lty=1,lwd=2,ylim=c(0,13))
xx <- PF.NClades$True.nfactors
lines(xx,predict(best.fit),lty='dashed',lwd=2)
polygon(c(rev(xx),xx),c(fitted[,'upr'],fitted[,'lwr']),col='grey80',border=NA)
lines(xx,fitted[,'lwr'],lty='dashed',col='red')
lines(xx,fitted[,'upr'],lty='dashed',col='red')
legend(1.3,12,legend=c('PhyloFactor Average','95% confidence intervals','True Value'),lty=c('dashed','dashed',1),col=c('black','red','black'),lwd=2)



par(mfrow=c(1,4))
for (cc in 1:length(stop.fcns)){
  C <- Clades[[cc]]
  boxplot(by(C[,1],C[,2],FUN=function(x){return(x)}),xlab='True K',ylim=c(1,10),ylab='PhyloFactor K*',pch=17,main=stop.fcns[[cc]])
  lines(c(1,N),c(1,N),lwd=2,col='blue')
  mns <- by(C[,1],C[,2],mean)
  pc <- sum(C[,2]==C[,1])/dim(C)[1]
  legend(x=1,y=9,legend='K=K*',lty=1,col='blue')
  legend(x=1,y=8,legend=paste('Percent K=K* : ',pc))
}


